from System import String
from System import DateTime

from System.IO import *
from System.Security import *
from System.Text import *

from Deadline.Scripting import *

import traceback

########################################################################
## Main Function Called By Deadline
########################################################################
def __main__( dlArgs, qsArgs ):
	returnVal = ""
	studioName = "StudioName"
	serverAddress = "localhost"
	serverPort = "8080"
	viewkeys = ["Name","UserName","Status","Priority","SubmitDateTime","TaskCount","CompletedChunks","ErrorReports","Group","Pool"]
	sb = StringBuilder()
	
	try:

		# Get the args
		egosort = qsArgs["ego"] if "ego" in qsArgs else None
		primarysort = qsArgs["psort"] if "psort" in qsArgs else None
		primarysortorder = qsArgs["psord"] if "psord" in qsArgs else None
		secondarysort = qsArgs["ssort"] if "ssort" in qsArgs else None
		secondarysortorder = qsArgs["ssord"] if "ssord" in qsArgs else None
		pulseaddress = qsArgs["pulse"] if "pulse" in qsArgs else None
		lastupdate = qsArgs["update"] if "update" in qsArgs else None
		
		# parse the last update
		if ( lastupdate != None ):
			lastupdate = fromPlatformIndDateTime( lastupdate )#BUG possibility when starting the app for the first time - unable to reproduce
		
		plugins = {}
		users = {}
		statuses = {}
		groups = {}
		pools = {}
		
		thisAddress = PulseUtils.GetPulseMacAddress()
		jobs = PulseUtils.GetPulseJobs()
		
		# do the sorts
		if ( secondarysort != None ):
			jobs = SortDictArrayByKey( jobs, secondarysort, secondarysortorder )
		if ( primarysort != None ):
			jobs = SortDictArrayByKey( jobs, primarysort, primarysortorder )
		if ( egosort != None ):
			jobs = EgoSortDictArray( jobs, egosort )
		
		sb.AppendLine( "<HTML>" )
		sb.AppendLine( '''
			<style>
			body
			{
				background: white;
				font-family: arial;
			}
			table, td, th
			{
				font-size:12px;
				border:1px solid black;
				border-collapse:collapse;
				margin:0px;
				padding:2px;
			}
			tr.alt td 
			{
				color:#000;
				background-color:#EEEEEE;
			}
			th
			{
				background-color:darkgray;
				color:white;
			}
			A:link {color:black}
			A:visited {color:black}
			A:active {color:gray}
			A:hover {color:gray}
			</style>'''
		)
		# jobs array
		sb.AppendLine( "<H1><B>%s</B></H1>" % (studioName))
		sb.AppendLine("<Table border=0>")
		sb.AppendLine("<TR>")
		for key in viewkeys:
			sb.AppendLine("<TH>")
			sb.AppendLine( "<B>%s</B>" % (key))
			sb.AppendLine("</TH>")
		sb.AppendLine("</TR>")	
		
		# add each job
		evenoddcount = 1
		for job in jobs:
			if ( job != None and job["Status"] != "Deleted"):
				
				# if there was no update time given, or the update time is before this jobs last update
				if ( lastupdate == None or pulseaddress != thisAddress or DateTime.Compare( lastupdate, lastJobUpdate ) < 0 ):
					sb.AppendLine("<TR class=%s>" % (evenoddcount % 2 and 'Alt' or ''))	
					for key in viewkeys:
						sb.AppendLine("<TD>")
						if (job["Status"]=="Active"):
							sb.AppendLine("<B>")
						if key == "Name":
							sb.AppendLine("<a href='%s'>" % ("http://"+serverAddress+":"+serverPort+"/getJob?"+job["JobId"]))
						sb.AppendLine( "%s" % (job[key]))
						if key == "Name":
							sb.AppendLine("</a>")
						sb.AppendLine("</TD>")
						if (job["Status"]=="Active"):
							sb.AppendLine("</B>")
					sb.AppendLine("</TR>")	
				# if the job is up to date, just print it's id to retrieve its data
				evenoddcount += 1 
		
		sb.AppendLine("</table>")
		sb.AppendLine( "</html>" )

		returnVal = sb.ToString()
		
	except Exception, detail:
		returnVal = "ERROR: %s" % detail
	
	return returnVal

def toPlatformIndDateTime( dt ):
	day = dt.Day
	month = dt.Month
	year = dt.Year
	
	second = dt.Second
	minute = dt.Minute
	hour = dt.Hour
	
	return str(day) + "/" + str(month) + "/" + str(year) + "_" + str(hour) + ":" + str(minute) + ":" + str(second)

# accepts "day/month/year hour:min:sec"
def fromPlatformIndDateTime( dt ):
	dt = str(dt)

	(date, time) = dt.split( "_" )
	(day, month, year) = date.split( "/" )
	(hour, minute, second) = time.split( ":" )
	
	return DateTime( int(year), int(month), int(day), int(hour), int(minute), int(second) )

def listKeys( title, dict, sb ):
	# print the dict
	sb.AppendLine( "\t\t<key>" + title + "</key>" )
	sb.AppendLine( "\t\t<dict>" )
	
	# print the values
	for key in dict:
		sb.AppendLine( "\t\t\t<key>" + key + "</key><true />" )
	
	# close the dict
	sb.AppendLine( "\t\t</dict>" )

def EgoSortDictArray( array, user ):
	userArray = []
	otherArray = []
	
	for i in range( 0, len(array) ):
		if ( String.Compare( array[i]["UserName"], user, True ) == 0 ):
			userArray.append( array[i] )
		else:
			otherArray.append( array[i] )
	
	for i in range( 0, len(otherArray) ):
		userArray.append( otherArray[i] )
	
	return userArray

def SortDictArrayByKey( array, key, order ):
    arrayList = []
    for item in array:
        arrayList.append(item)

	return mergesort( arrayList, key, order )
	
def mergesort( list, key, order ):
	# base case
	if ( len( list ) <= 1 ):
		return list
	
	# get left and right list
	left = list[:len(list)/2]
	right = list[len(list)/2:len(list)]
	
	# recursively sort the list
	left = mergesort( left, key, order )
	right = mergesort( right, key, order )
	
	# merge the lists into ret
	return merge( left, right, key, order )

def merge( left, right, key, order ):
	# init return list
	ret = []
	
	# init position in left and right lists
	leftPos = 0
	rightPos = 0
	
	# get the length of the two lists
	lenLeft = len( left )
	lenRight = len( right )
	
	# merge the lists
	if ( order == None or order == "0" ):
		while ( leftPos < lenLeft and rightPos < lenRight ):
			if ( String.Compare( key, "SubmitDateTime", True ) != 0 ):
				# Normal String Compare
				if ( left[leftPos] == None ):
					leftPos += 1
				elif ( right[rightPos] == None ):
					rightPos += 1
				elif ( String.Compare( left[leftPos][key], right[rightPos][key], True ) <= 0 ):
					ret.append( left[leftPos] )
					leftPos += 1
				else:
					ret.append( right[rightPos] )
					rightPos += 1
			else:
				# Compare Dates
				if ( left[leftPos] == None ):
					leftPos += 1
				elif ( right[rightPos] == None ):
					rightPos += 1
				elif ( DateTime.Compare( DateTime.Parse( left[leftPos][key] ), DateTime.Parse( right[rightPos][key] ) ) >= 0 ):
					ret.append( left[leftPos] )
					leftPos += 1
				else:
					ret.append( right[rightPos] )
					rightPos += 1
	else:
		while ( leftPos < lenLeft and rightPos < lenRight ):
			if ( String.Compare( key, "SubmitDateTime", True ) != 0 ):
				# Normal String Compare
				if ( left[leftPos] == None ):
					leftPos += 1
				elif ( right[rightPos] == None ):
					rightPos += 1
				elif ( String.Compare( left[leftPos][key], right[rightPos][key], True ) >= 0 ):
					ret.append( left[leftPos] )
					leftPos += 1
				else:
					ret.append( right[rightPos] )
					rightPos += 1
			else:
				# Compare Dates
				if ( left[leftPos] == None ):
					leftPos += 1
				elif ( right[rightPos] == None ):
					rightPos += 1
				elif ( DateTime.Compare( DateTime.Parse( left[leftPos][key] ), DateTime.Parse( right[rightPos][key] ) ) <= 0 ):
					ret.append( left[leftPos] )
					leftPos += 1
				else:
					ret.append( right[rightPos] )
					rightPos += 1
	
	# extend ret with the remaining list
	if ( leftPos < lenLeft ):
		while ( leftPos < lenLeft ):
			if ( left[leftPos] == None ):
				leftPos += 1
			else:
				ret.append( left[leftPos] )
				leftPos += 1
	else:
		while ( rightPos < lenRight ):
			if ( right[rightPos] == None ):
				rightPos += 1
			else:
				ret.append( right[rightPos] )
				rightPos += 1
	
	return ret